/************************************************************************

  	PIC Etherfeed - tickling your ego or statistics? If you have a
	website, blog or any other web service than the etherfeed is for you.
	The etherfeed is a small standalone network device, it connects to the
	Internet and displays statistical data about your website like number
	of unique visitors, number of pageviews, PayPal sales (SUM of sales via
	the PayPal IPN), Facebook likes for your brand / group and weather data
	like temperature, humidity for your location.
	
	OpenSource & OpenHardware project with kits available at:

	http://www.etherfeed.com
	https://www.facebook.com/etherfeed

    Copyright (C) 2012  Boris Ribov, Georgi Bakalski, Militsa Ilieva

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 *********************************************************************
 *
 *  Main Application of Etherfeed with TCP/IP
 * 	FileName: Etherfeed.c
 *********************************************************************
 * Boris Ribov   01/07/2012    Rev. 2.0		; Base version of the project
 ********************************************************************/

// * This macro uniquely defines this file as the main entry point.
// * There should only be one such definition in the entire project,
// * and this file must define the AppConfig variable as described below.
#define THIS_IS_STACK_APPLICATION

// Include all headers for any enabled TCPIP Stack functions
#include "TCPIP Stack/TCPIP.h"

// Include functions specific to this stack application
#include "Etherfeed.h"
#include "Modules\Time.h"
#include "Modules\LCD.h"
#include "Modules\Client.h"

// Declare AppConfig structure and some other supporting stack
// variables. This structure is defined in StackTask.h
APP_CONFIG AppConfig;

// System local variables for the system time
DWORD upTime = 0;					// UpTime counter in seconds
DWORD sysTime = 1262347200;			// System time in seconds starting at 01-JAN-2010, 12am:00
BYTE SNTP_for_the_first_time = 0; 	// Display if the NTP is firstly sync

// To interconnect to WEB Client for the external server script
extern TICK sample_time;
			
// Private helper functions.
static void InitAppConfig(void);
static void InitializeBoard(void);
static void EnableInterrupts(void);
static void ProcessIO(void);
void DisplayIPValue(IP_ADDR IPVal);

// PIC18 Interrupt Service Routines
#pragma interruptlow LowISR
void LowISR(void)
{
    TickUpdate();
}
#pragma interrupt HighISR
void HighISR(void)
{
}

#pragma code lowVector=0x18
void LowVector(void){_asm goto LowISR _endasm}

#pragma code highVector=0x8
void HighVector(void){_asm goto HighISR _endasm}

// Return to default code section
#pragma code 

// Main application entry point.
void main(void)
{
    static TICK t_up, t_sntp = 0;
	static DWORD dwLastIP = 0;

    // Initialize application specific hardware
    InitializeBoard();

	// Initialize LCD
	LCDInit();
	strcpypgm2ram (LCDText, "Etherfeed Rev. 2");
	LCDUpdate ();

	// Initialize stack-related hardware components that may be 
    TickInit();

	// File system init
	MPFSInit();

	// Initialize Stack and application related NV variables into AppConfig.
	InitAppConfig();

	// Initialize core stack layers (MAC, ARP, TCP, UDP) and application modules (HTTP, SNMP, etc.)
    StackInit();
	
	// Set the time in system time
	Init_Time(AppConfig.TZ, 0);

	// Enable system clock interrupt
	EnableInterrupts ();

    // Initiates board setup process if button is depressed on startup
    if(RESET_IO == 0u)
    {
		// Invalidate the EEPROM contents if BUTTON0 is held down for more than 4 seconds
		DWORD StartTime = TickGet();				
		while(RESET_IO == 0u)
		{
			if(TickGet() - StartTime > 4*TICK_SECOND)
			{
			    XEEBeginWrite(0x0000);
			    XEEWrite(0xFF);
			    XEEEndWrite();

				strcpypgm2ram (LCDText, "Reset completed!");
				LCDUpdate ();

				while((LONG)(TickGet() - StartTime) <= (LONG)(3*TICK_SECOND));
				while(RESET_IO == 0u);
				Reset();
				break;
			}
		}
	}

	// Load the current time for WEB client timeout checking
	sample_time = TickGet();

	// MAIN WHILE LOOP
    while(1)
    {
        // Increase system time on every second.
        if(TickGet() - t_up >= TICK_SECOND)
        {
            t_up = TickGet();

			// Update system time counters
			upTime++;
			sysTime++;
        }

		// Update the system time with SNTP time on every 12 hours
		if (SNTPGetUTCSeconds() > 1233489600)
		{
        	if((TickGet() - t_sntp >= 12*3600*TICK_SECOND) || (SNTP_for_the_first_time == 0))
			{
			    t_sntp = TickGet();

				sysTime = SNTPGetUTCSeconds();
				SNTP_for_the_first_time = 1;
			}
		} 

  
        // This task performs normal stack task including checking
        // for incoming packet, type of packet and calling
        // appropriate stack entity to process it.
        StackTask();
        
        // This tasks invokes each of the core stack application tasks
        StackApplications();
		
		// Run extended TCP WEB Client application
		TCPClient ();

		// User process
		ProcessIO();

        // If the local IP address has changed (ex: due to DHCP lease change)
		if((dwLastIP != AppConfig.MyIPAddr.Val) && (upTime > 5))
		{
			dwLastIP = AppConfig.MyIPAddr.Val;
			DisplayIPValue(AppConfig.MyIPAddr);

			sample_time = TickGet();
			#if defined(STACK_USE_ANNOUNCE)
				AnnounceIP();
			#endif
		}
	}// END OF MAIN LOOP	
}

//******************************************************
// User process
//******************************************************
static void ProcessIO(void)
{
}

/*********************************************************************
 * Function:        void InitializeBoard(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Initialize board specific hardware.
 * Note:            None
 ********************************************************************/
static void InitializeBoard(void)
{	
	// Enable 4x/5x PLL on PIC18F87J10, PIC18F97J60, etc.
    OSCTUNE = 0x40;
    // Set direction and initial default valuefor GPIO Ports
    LATA  = INIT_PORTA;
    TRISA = INIT_TRISA;
    LATB  = INIT_PORTB;
    TRISB = INIT_TRISB;
    LATC  = INIT_PORTC;
    TRISC = INIT_TRISC;
    LATD  = INIT_PORTD;
    TRISD = INIT_TRISD;
    LATE  = INIT_PORTE;
    TRISE = INIT_TRISE;
    LATF  = INIT_PORTF;
    TRISF = INIT_TRISF;
    LATG  = INIT_PORTG;
    TRISG = INIT_TRISG;
    // Disable internal PORTB pull-ups
    INTCON2bits.RBPU = 1;
    // Set up analog features of PORTA
    ADCON0 = INIT_ADCON0; 
    ADCON1 = INIT_ADCON1;
	ADCON2 = INIT_ADCON2;
	// Disable comparators
	CMCON = INIT_CMCON;
}

/*********************************************************************
 * Function:        void EnableInterrupts(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Initialize board specific hardware.
 * Note:            None
 ********************************************************************/
static void EnableInterrupts(void)
{
	// Timer 1 interrupts
	IPR1bits.TMR1IP = 1;	// High Priority
	PIE1bits.TMR1IE = 0;	// Disable Timer 1 interrupt
	// USART TRANSMITTER interrupt
	IPR1bits.RCIP = 0;		// Low priority reciever
	IPR1bits.TXIP = 0;		// Low priority transmiter
	PIE1bits.RCIE = 0; 		// Disable USART RECIEVER interrupt
	PIE1bits.TXIE = 0; 		// Disable UART transmitter
	// Enable Interrupts
	RCONbits.IPEN = 1;		// Enable interrupt priorities
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;
}


// Uncomment these two pragmas for production MAC address 
// serialization if using C18. The MACROM=0x1FFF0 statement causes 
// the MAC address to be located at aboslute program memory address 
// 0x1FFF0 for easy auto-increment without recompiling the stack for 
// each device made.  Note, other compilers/linkers use a different 
// means of allocating variables at an absolute address.  Check your 
// compiler documentation for the right method.

//#pragma romdata MACROM=0x1FFF0
static ROM BYTE SerializedMACAddress[6] = {MY_DEFAULT_MAC_BYTE1, MY_DEFAULT_MAC_BYTE2, MY_DEFAULT_MAC_BYTE3, MY_DEFAULT_MAC_BYTE4, MY_DEFAULT_MAC_BYTE5, MY_DEFAULT_MAC_BYTE6};
//#pragma romdata

/*********************************************************************
 * Function:        void InitAppConfig(void)
 * PreCondition:    MPFSInit() is already called.
 * Input:           None
 * Output:          Write/Read non-volatile config variables.
 * Side Effects:    None
 * Overview:        None
 * Note:            None
 ********************************************************************/
static void InitAppConfig(void)
{
    BYTE c;

	AppConfig.Flags.bIsDHCPEnabled = FALSE;
	AppConfig.Flags.bInConfigMode = TRUE;
	memcpypgm2ram((void*)&AppConfig.MyMACAddr, (ROM void*)SerializedMACAddress, sizeof(AppConfig.MyMACAddr));
	//	{
	//		_prog_addressT MACAddressAddress;
	//		MACAddressAddress.next = 0x157F8;
	//		_memcpy_p2d24((char*)&AppConfig.MyMACAddr, MACAddressAddress, sizeof(AppConfig.MyMACAddr));
	//	}
	AppConfig.MyIPAddr.Val = MY_DEFAULT_IP_ADDR_BYTE1 | MY_DEFAULT_IP_ADDR_BYTE2<<8ul | MY_DEFAULT_IP_ADDR_BYTE3<<16ul | MY_DEFAULT_IP_ADDR_BYTE4<<24ul;
	AppConfig.DefaultIPAddr.Val = AppConfig.MyIPAddr.Val;
	AppConfig.MyMask.Val = MY_DEFAULT_MASK_BYTE1 | MY_DEFAULT_MASK_BYTE2<<8ul | MY_DEFAULT_MASK_BYTE3<<16ul | MY_DEFAULT_MASK_BYTE4<<24ul;
	AppConfig.DefaultMask.Val = AppConfig.MyMask.Val;
	AppConfig.MyGateway.Val = MY_DEFAULT_GATE_BYTE1 | MY_DEFAULT_GATE_BYTE2<<8ul | MY_DEFAULT_GATE_BYTE3<<16ul | MY_DEFAULT_GATE_BYTE4<<24ul;
	AppConfig.PrimaryDNSServer.Val = MY_DEFAULT_PRIMARY_DNS_BYTE1 | MY_DEFAULT_PRIMARY_DNS_BYTE2<<8ul  | MY_DEFAULT_PRIMARY_DNS_BYTE3<<16ul  | MY_DEFAULT_PRIMARY_DNS_BYTE4<<24ul;
	AppConfig.SecondaryDNSServer.Val = MY_DEFAULT_SECONDARY_DNS_BYTE1 | MY_DEFAULT_SECONDARY_DNS_BYTE2<<8ul  | MY_DEFAULT_SECONDARY_DNS_BYTE3<<16ul  | MY_DEFAULT_SECONDARY_DNS_BYTE4<<24ul;
	// Load the default NetBIOS Host Name
	memcpypgm2ram(AppConfig.NetBIOSName, (ROM void*)MY_DEFAULT_HOST_NAME, 16);
	FormatNetBIOSName(AppConfig.NetBIOSName);
	//Load the password
	memcpypgm2ram(AppConfig.Password, (ROM void*)MY_DEFAULT_PASSWORD, 9);
	//Load the time zone params
	memcpypgm2ram(AppConfig.UTC_Serv, (ROM void*)MY_DEFAULT_UTC_SERV, 21);
	AppConfig.TZ = MY_DEFAULT_TZ;
	//Load external script patams
	memcpypgm2ram(AppConfig.Scr_Svr, (ROM void*)MY_DEFAULT_SVR_ADDR, 21);
	memcpypgm2ram(AppConfig.Scr_Path, (ROM void*)MY_DEFAULT_SRV_PATH, 21);
	AppConfig.Scr_Refresh = MY_DEFAULT_SRV_REF;
    // When a record is saved, first byte is written as 0x61 to indicate
    // that a valid record was saved.
	XEEReadArray(0x0000, &c, 1);
    if(c == 0x61u)
	    XEEReadArray(0x0001, (BYTE*)&AppConfig, sizeof(AppConfig));
    else
        SaveAppConfig();
}

// ***********************************************************************
// Subroutine to save the APP CONFIG structure to the EEPROM
// ***********************************************************************
void SaveAppConfig(void)
{
	// Ensure adequate space has been reserved in non-volatile storage to 
	// store the entire AppConfig structure.  If you get stuck in this while(1) 
	// trap, it means you have a design time misconfiguration in TCPIPConfig.h.
	// You must increase MPFS_RESERVE_BLOCK to allocate more space.
	#if defined(STACK_USE_MPFS) || defined(STACK_USE_MPFS2)
		if(sizeof(AppConfig) > MPFS_RESERVE_BLOCK)
			while(1);
	#endif

    XEEBeginWrite(0x0000);
    XEEWrite(0x61);
    XEEWriteArray((BYTE*)&AppConfig, sizeof(AppConfig));
}

// ***********************************************************************
// Subroutine to writes an IP address to the LCD display
// ***********************************************************************
void DisplayIPValue(IP_ADDR IPVal)   
{     
    BYTE IPDigit[4];   
    BYTE i;   
    BYTE j;   
    BYTE LCDPos=0;   

	LCDErase ();
    for(i = 0; i < sizeof(IP_ADDR); i++)   
    {   
        uitoa((WORD)IPVal.v[i], IPDigit);   
    
        for(j = 0; j < strlen((char*)IPDigit); j++)   
        {   
          LCDText[LCDPos++] = IPDigit[j];   
        }   
        if(i == sizeof(IP_ADDR)-1)   
          break;   
        LCDText[LCDPos++] = '.';    
    }   
    LCDUpdate();   
}