/************************************************************************

  	PIC Etherfeed - tickling your ego or statistics? If you have a
	website, blog or any other web service than the etherfeed is for you.
	The etherfeed is a small standalone network device, it connects to the
	Internet and displays statistical data about your website like number
	of unique visitors, number of pageviews, PayPal sales (SUM of sales via
	the PayPal IPN), Facebook likes for your brand / group and weather data
	like temperature, humidity for your location.
	
	OpenSource & OpenHardware project with kits available at:

	http://www.etherfeed.com
	https://www.facebook.com/etherfeed

    Copyright (C) 2012  Boris Ribov, Georgi Bakalski, Militsa Ilieva

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 *********************************************************************
 *
 *  Generic TCP Client for the external scrypt communication 
 * 	FileName: Client.c
 *
 *********************************************************************
 * Boris Ribov   01/07/2012    Rev. 2.0		; 
 ********************************************************************/
#include "TCPIP Stack/TCPIP.h"
#include "LCD.h"

// Received text buffer
BYTE DataBuff[80];

// Base remote server port
WORD ServerPort = 80;

// Some FLAG variables
// 1 - server error; 0 - server success
BYTE conn_flag 		= 1;	

// Some temporary variables
BYTE cntr	 		= 0;
BYTE counter		= 0;
BYTE header_state	= 0;
TICK sample_time 	= 0;

/******************************************************************************
 * Function:        void GenericTCPClient(void)
 * PreCondition:    Stack is initialized()
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        None
 * Note:            None
 ******************************************************************************/
void TCPClient(void)
{
    BYTE i, *StringPtr;
	static TICK timeout_cnt;
    static TCP_SOCKET MySocket = INVALID_SOCKET;
    static NODE_INFO Server;
    static enum _GenericTCPExampleState
    {
        SM_HOME = 0,
        SM_SOCKET_OBTAINED,
        SM_PROCESS_RESPONSE,
        SM_DISCONNECT,
        SM_DONE
    } GenericTCPExampleState = SM_DONE;

    switch(GenericTCPExampleState)
    {
        case SM_HOME:
			conn_flag = 1;
			timeout_cnt = TickGet();

			MySocket = TCPOpen((DWORD)&AppConfig.Scr_Svr[0], TCP_OPEN_RAM_HOST, 80, TCP_PURPOSE_DEFAULT);
    		//MySocket = TCPOpen((DWORD)(PTR_BASE)"www.irdroid.com", TCP_OPEN_ROM_HOST, 80, TCP_PURPOSE_DEFAULT);

           	GenericTCPExampleState = SM_SOCKET_OBTAINED;
			break;

        case SM_SOCKET_OBTAINED:

        	if(TickGet() - timeout_cnt > 4*TICK_SECOND)
            {
             	// Time out if too much time is spent in this state
	      		GenericTCPExampleState = SM_DISCONNECT;
            }

            // Wait for the remote server to accept our connection request
            if(!TCPIsConnected(MySocket)) 
                break;

            // Make certain the socket can be written to
            if(!TCPIsPutReady(MySocket))
                break;
            
            // Place the application protocol data into the transmit buffer.
            // For this example, we are connected to an HTTP server, so we'll
            // send an HTTP GET request.
            TCPPut(MySocket, 'G');
            TCPPut(MySocket, 'E');
            TCPPut(MySocket, 'T');
            TCPPut(MySocket, ' ');
            {
				// Use the request name for the GET
                StringPtr = AppConfig.Scr_Path;
                for(i = 0; i < strlen(AppConfig.Scr_Path); i++)
                {
                    TCPPut(MySocket, *StringPtr++);
                }
            }
            TCPPut(MySocket, ' ');
            TCPPut(MySocket, 'H');
            TCPPut(MySocket, 'T');
            TCPPut(MySocket, 'T');
            TCPPut(MySocket, 'P');
            TCPPut(MySocket, '/');
            TCPPut(MySocket, '1');
            TCPPut(MySocket, '.');
            TCPPut(MySocket, '1');
            TCPPut(MySocket, '\r');
            TCPPut(MySocket, '\n');
            TCPPut(MySocket, 'H');
            TCPPut(MySocket, 'o');
            TCPPut(MySocket, 's');
            TCPPut(MySocket, 't');
            TCPPut(MySocket, ':');
            TCPPut(MySocket, ' ');
            {
                StringPtr = AppConfig.Scr_Svr;
                for(i = 0; i < strlen(AppConfig.Scr_Svr); i++)
                {
                    TCPPut(MySocket, *StringPtr++);
                } 
            }
            TCPPut(MySocket, '\r');
            TCPPut(MySocket, '\n');
            TCPPut(MySocket, '\r');
            TCPPut(MySocket, '\n');

            // Send the packet
            TCPFlush(MySocket);

            GenericTCPExampleState = SM_PROCESS_RESPONSE;
			// ready for reply comparison for successful data upload
			cntr = 0;
			break;

        case SM_PROCESS_RESPONSE:
			// Check to see for the receive timeout
        	if(TickGet() - timeout_cnt > 8*TICK_SECOND)
			{
				GenericTCPExampleState = SM_DISCONNECT;
				break;
			}
			
            // Check to see if the remote node has disconnected from us or sent data
            if(!TCPIsConnected(MySocket))
            {
                GenericTCPExampleState = SM_DISCONNECT;
            }

            if(!TCPIsGetReady(MySocket))
                break;

            // Obtain the server reply
			counter = 0;
			header_state = 0;
            while(TCPGet(MySocket, &i))
            {  

				// Discard the HTML header
				// Searching for the 0x0D,0x0A,0x0D,0x0A
				if (header_state == 0)
				{
					switch (counter)
					{
						case 0:
							if (i == 0x0D)
								counter++;					
							break;
						case 1:
							if (i == 0x0A)
								counter++;
							else
								counter=0;
							break;
						case 2:
							if (i == 0x0D)
								counter++;
							else
								counter=0;
							break;
						case 3:
							if (i == 0x0A)
							{
								counter = 0;
								header_state = 1;
								continue;
							}
							else
								counter=0;
							break;
					}
				}

				// Discard the payload size checking
				if (header_state == 1)
				{
					switch (counter)
					{
						case 0:
							if (i == 0x0D)
								counter++;					
							break;
						case 1:
							if (i == 0x0A)
							{
								counter = 0;
								header_state = 2;
								continue;
							}
							else
								counter=0;
							break;
					}
				}

				// Discard the HTML tag
				if ((counter < 4) && (header_state == 0))
					continue;
				// Discard the HTML body size
				if ((counter < 2) && (header_state == 1))
					continue;

				// End of data
				if (i == 0x0D)
				{
					GenericTCPExampleState = SM_DISCONNECT;	
					conn_flag = 0;
					DataBuff[cntr] = 0;					
					LCDErase ();
					strcpy (LCDText, DataBuff);
					LCDUpdate ();
					break;
				}

				// Limit the answer to 16 chars
				if (cntr > 15)
				{
					GenericTCPExampleState = SM_DISCONNECT;	
					conn_flag = 0;
					DataBuff[cntr] = 0;
					LCDErase ();
					strcpy (LCDText, DataBuff);
					LCDUpdate ();
					break;
				}

				// Reading the server data
				//if (i == 0xBA) i = 0xDF;
				DataBuff[cntr] = i;
				cntr++;
            }
            break;

        case SM_DISCONNECT:
            // Close the socket so it can be used by other modules
            // For this application, we wish to stay connected, but this state will
            // still get entered if the remote server decides to disconnect
            TCPFlush(MySocket);
            TCPDisconnect(MySocket);
            MySocket = INVALID_SOCKET;
            GenericTCPExampleState = SM_DONE;
            break;

        case SM_DONE:
			// Get new reply after the required timeout
       		if(TickGet() - sample_time > AppConfig.Scr_Refresh*TICK_SECOND)
			{
				sample_time = TickGet();			// Get the old system time for the 60sec interval bw the samples
				GenericTCPExampleState = SM_HOME;
			}
            break;
    }
}

